package ru.dbotthepony.kstarbound.world

import ru.dbotthepony.kstarbound.io.readVec2i
import ru.dbotthepony.kstarbound.io.writeVec2i
import ru.dbotthepony.kvector.api.IStruct2d
import ru.dbotthepony.kvector.api.IStruct2f
import ru.dbotthepony.kvector.api.IStruct2i
import ru.dbotthepony.kvector.vector.Vector2i
import java.io.DataInputStream
import java.io.DataOutputStream

data class WorldGeometry(val size: Vector2i, val loopX: Boolean, val loopY: Boolean) {
	constructor(buff: DataInputStream) : this(buff.readVec2i(), buff.readBoolean(), buff.readBoolean())

	val x: CoordinateMapper = if (loopX) CoordinateMapper.Wrapper(size.x) else CoordinateMapper.Clamper(size.x)
	val y: CoordinateMapper = if (loopY) CoordinateMapper.Wrapper(size.y) else CoordinateMapper.Clamper(size.y)

	fun write(buff: DataOutputStream) {
		buff.writeVec2i(size)
		buff.writeBoolean(loopX)
		buff.writeBoolean(loopY)
	}

	fun chunkFromCell(pos: IStruct2i): ChunkPos {
		return ChunkPos(x.chunkFromCell(pos.component1()), y.chunkFromCell(pos.component2()))
	}

	fun chunkFromCell(pos: IStruct2f): ChunkPos {
		return ChunkPos(x.chunkFromCell(pos.component1()), y.chunkFromCell(pos.component2()))
	}

	fun chunkFromCell(pos: IStruct2d): ChunkPos {
		return ChunkPos(x.chunkFromCell(pos.component1()), y.chunkFromCell(pos.component2()))
	}

	fun wrap(pos: ChunkPos): ChunkPos {
		val x = this.x.chunk(pos.x)
		val y = this.y.chunk(pos.y)
		if (x == pos.x && y == pos.y) return pos
		return ChunkPos(x, y)
	}

	fun wrapToLong(pos: ChunkPos): Long {
		val x = this.x.chunk(pos.x)
		val y = this.y.chunk(pos.y)
		if (x == pos.x && y == pos.y) return pos.toLong()
		return ChunkPos.toLong(x, y)
	}
}
